<?php

namespace EventPortal\Domain\Agenda\Models;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;

class Pathway extends Model
{
    public const KEY = 'pathway';
    public const DEFAULT = 'Default';

    /** @var array */
    protected $guarded = [];

    public static function generate(Event $event, string $name, string $description): Pathway
    {
        return Pathway::create([
            'event_id' => $event->id,
            'name' => $name,
            'description' => $description,
        ]);
    }

    public function sessions(): HasMany
    {
        return $this->hasMany(Session::class);
    }

    public function event(): BelongsTo
    {
        return $this->belongsTo(Event::class);
    }

    public function eventDelegates(): HasMany
    {
        return $this->hasMany(EventDelegate::class, 'pathway_id');
    }

    public function scopeDefaultForEvent(Builder $query, Event $event): Builder
    {
        return $query
            ->where('name', self::DEFAULT)
            ->where('event_id', $event->id);
    }

    public function scopeNonDefaultForEventDelegate(Builder $query, EventDelegate $eventDelegate): Builder
    {
        return $query
            ->where('name', '!=', self::DEFAULT)
            ->where('id', $eventDelegate->pathway_id);
    }
}
