<?php

namespace EventPortal\Domain\FormProgress;

use EventPortal\Domain\Delegate\Models\DelegateFormStep;
use EventPortal\Domain\Event\BroadcastCallToAction;
use EventPortal\Domain\Event\DeclineCallToAction;
use EventPortal\Domain\Event\Form\Enums\State;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\Event\RegistrationCallToAction;
use EventPortal\Domain\FormProgress\FormStep as FormStepObject;

class FormProgress
{
    public string $title;

    public array $actions;

    public array $progress;

    private ?Form $form;

    private EventDelegate $eventDelegate;

    public static function make(): self
    {
        return new self();
    }

    public function forForm(?Form $form = null): self
    {
        $this->form = $form;

        return $this;
    }

    public function forEventDelegate(EventDelegate $eventDelegate): self
    {
        $this->eventDelegate = $eventDelegate;

        return $this;
    }

    public function get(): self
    {
        if ($this->form === null) {
            return $this;
        }

        $this->title = $this->form->name;
        $this->progress = $this->getProgress();

        $actions = [
            BroadcastCallToAction::make()->forEventDelegate($this->eventDelegate)->get(),
            RegistrationCallToAction::make()->forEventDelegate($this->eventDelegate)->get(),
            DeclineCallToAction::make()->forEventDelegate($this->eventDelegate)->get(),
        ];

        $this->actions = array_values(array_filter($actions));

        return $this;
    }

    public function toArray(): ?array
    {
        if ($this->form === null) {
            return null;
        }

        return [
            'title' => $this->title,
            'progress' => $this->progress,
            'actions' => $this->actions,
        ];
    }

    private function getProgress(): array
    {
        $formSteps = FormStep::where('form_id', $this->form->id)->get();

        $delegateFormSteps = DelegateFormStep::query()
            ->whereIn('form_step_id', $formSteps->pluck('id'))
            ->where('delegate_id', $this->eventDelegate->delegate->id)
            ->get();

        $progress = $formSteps
            ->mapInto(FormStepObject::class)
            ->map(function (FormStepObject $value) use ($delegateFormSteps) {
                $delegateFormStep = $delegateFormSteps->firstWhere('form_step_id', $value->formStepId);

                if ($delegateFormStep !== null) {
                    $value->completed = $delegateFormStep->state->getValue() === State::Complete()->getValue();
                }

                return $value;
            });

        return $progress->map->toArray()->toArray();
    }
}
