<?php

namespace EventPortal\Domain\Util;

use EventPortal\Domain\Component\Option;
use Illuminate\Support\Arr;

class Status
{
    /** @var array */
    private $config;

    public function __construct(string $key)
    {
        $this->config = config("state-machine.{$key}.states", []);
    }

    public static function make(string $key): self
    {
        return new self($key);
    }

    public function asOptions(): array
    {
        return collect($this->config)
            ->map(function ($status) {
                return Option::make($status['metadata']['title'])->value($status['name']);
            })
            ->toArray();
    }

    public function defaultOptions(): array
    {
        return collect($this->config)
            ->filter(function ($status) {
                return Arr::get($status, 'metadata.admin.showByDefault', true);
            })
            ->map(function ($status) {
                return $status['name'];
            })
            ->values()
            ->toArray();
    }

    public function forFilters(): array
    {
        return collect($this->config)
            ->map(function ($status) {
                return Option::make($status['metadata']['title'])->value($status['name']);
            })
            ->values()
            ->toArray();
    }

    public function getReadable(string $status): string
    {
        $states = Arr::where($this->config, function ($state) use ($status) {
            return $state['name'] === $status;
        });

        return Arr::get(head($states), 'metadata.title');
    }

    public function getDescription(string $status): ?string
    {
        $states = Arr::where($this->config, function ($state) use ($status) {
            return $state['name'] === $status;
        });

        return Arr::get(head($states), 'metadata.description');
    }

    public function getAlert(string $status): ?string
    {
        $states = Arr::where($this->config, function ($state) use ($status) {
            return $state['name'] === $status;
        });

        return Arr::get(head($states), 'metadata.alert');
    }
}
