<?php

namespace EventPortal\Http\Controllers\Delegate\Event;

use EventPortal\Contracts\Event;
use EventPortal\Domain\Event\Form\Enums\FormType;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\Event\ValueObjects\PageHeader;
use EventPortal\Domain\EventDelegate\Renderers\LandingRenderer;
use EventPortal\Domain\FormProgress\FormProgress;
use EventPortal\Http\Controllers\Controller;
use Inertia\Response;

class LandingController extends Controller
{
    public function __invoke(Event $event): Response
    {
        $delegate = auth()->user()->delegate;
        $eventDelegate = EventDelegate::forEventAndDelegate($event, $delegate)->firstOrFail();

        $pageHeader = PageHeader::make()
            ->title($event->name)
            ->description($event->snippet)
            ->date($event->readable_dates);

        $form = Form::whereType(FormType::Registration())->where('event_id', $event->id)->first();

        $formProgress = FormProgress::make()
            ->forForm($form)
            ->forEventDelegate($eventDelegate)
            ->get();

        return LandingRenderer::component('Delegate/Landing/Index')
            ->withPageHeader($pageHeader)
            ->withFormProgress($formProgress)
            ->withStartDate(optional($event)->readable_start_date)
            ->withAbout($event->description)
            ->withSpeakers($event->speakers)
            ->withVenues($event->venues)
            ->get();
    }
}
