<?php

namespace EventPortal\Tests\Unit\Approval\Actions;

use EventPortal\Domain\Approval\Models\Approval;
use EventPortal\Domain\Approval\Actions\TransitionEventApprovalStatus;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use SM\SMException;

class TransitionEventApprovalTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_transition()
    {
        $eventApproval = factory(Approval::class)->state('awaiting-approval')->create();

        (new TransitionEventApprovalStatus())->execute($eventApproval, 'line-manager-approved');

        $this->assertEquals('line-manager-approved', $eventApproval->status);
    }

    /** @test **/
    public function cannot_transition_to_nonexistent_state()
    {
        $this->expectException(SMException::class);

        $eventApproval = factory(Approval::class)->state('awaiting-approval')->create();

        (new TransitionEventApprovalStatus())->execute($eventApproval, 'non-existent-state');

        $this->assertEquals('awaiting', $eventApproval->status);
    }

    /** @test **/
    public function cannot_transition_to_state_that_is_not_allowed()
    {
        $this->expectException(SMException::class);

        $eventApproval = factory(Approval::class)->state('approved')->create();

        (new TransitionEventApprovalStatus())->execute($eventApproval, 'declined');

        $this->assertEquals('approved', $eventApproval->status);
    }
}
