<?php

namespace EventPortal\Tests\Unit\Event\Form\Actions;

use EventPortal\Domain\Event\Form\Actions\UpdateValidations;
use EventPortal\Domain\Event\Form\Models\Question;
use EventPortal\Domain\Event\Form\Models\QuestionValidation;
use EventPortal\Domain\Event\Form\ValueObjects\ValidationData;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Collection;

class UpdateValidationsTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_update_form_validation(): void
    {
        $question = factory(Question::class)->create();
        factory(QuestionValidation::class)->create(['question_id' => $question->id]);

        (new UpdateValidations)->execute(
            $this->getUpdatedValidation(),
            $question
        );

        $validation = $question->validations->first();
        $this->assertEquals('Updated test rule', $validation->rule);
        $this->assertEquals('Updated test message', $validation->message);
    }

    private function getUpdatedValidation(array $override = []): Collection
    {
        return collect(array_merge([
            [
               'id' => 1,
               'rule' => 'Updated test rule',
               'message' => 'Updated test message',
            ],
        ], $override))->mapInto(ValidationData::class);
    }

    /** @test **/
    public function can_add_new_form_validation(): void
    {
        $question = factory(Question::class)->create();
        factory(QuestionValidation::class)->create(['question_id' => $question->id]);

        (new UpdateValidations)->execute(
            $this->getNewValidation(),
            $question
        );

        $validation = $question->validations->first();
        $this->assertEquals('New test rule', $validation->rule);
        $this->assertEquals('New test message', $validation->message);
    }

    private function getNewValidation(array $override = []): Collection
    {
        return collect(array_merge([
            [
                'rule' => 'New test rule',
                'message' => 'New test message',
            ],
        ], $override))->mapInto(ValidationData::class);
    }

    /** @test **/
    public function deletes_unused_form_validations(): void
    {
        $question = factory(Question::class)->create();
        factory(QuestionValidation::class)->create(['question_id' => $question->id]);

        (new UpdateValidations)->execute(
            collect([]),
            $question
        );

        $this->assertCount(0, $question->validations);
    }

    /** @test **/
    public function adds_required_validation_if_needed(): void
    {
        $question = factory(Question::class)->create();
        factory(QuestionValidation::class)->create(['question_id' => $question->id]);

        (new UpdateValidations)->execute(
            collect([]),
            $question,
            true
        );

        $validation = $question->validations->first();
        $this->assertEquals('required', $validation->rule);
        $this->assertEquals('This field is required', $validation->message);
    }
}
