<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Util\Status;
use EventPortal\Tests\TestCase;

class StatusTest extends TestCase
{
    /** @test **/
    public function can_get_as_options()
    {
        $expected = [
            Option::make('Draft'),
            Option::make('Published'),
            Option::make('Post'),
            Option::make('Cancelled'),
            Option::make('Archived'),
        ];

        $this->assertEquals($expected, Status::make('event')->asOptions());
    }

    /** @test **/
    public function can_get_for_filters()
    {
        $expected = [
            Option::make('Draft'),
            Option::make('Published'),
            Option::make('Post'),
            Option::make('Cancelled'),
            Option::make('Archived'),
        ];

        $this->assertEquals($expected, Status::make('event')->forFilters());
    }

    /** @test **/
    public function can_get_readable_version_of_status()
    {
        $this->assertEquals('Published', Status::make('event')->getReadable('published'));
        $this->assertEquals('Cancelled', Status::make('event')->getReadable('cancelled'));
    }

    /** @test **/
    public function can_get_description_of_status()
    {
        $this->assertEquals(
            'Your event is awaiting approval.',
            Status::make('novoVisionMiff')->getDescription('awaiting-approval')
        );

        $this->assertNull(Status::make('novoVisionMiff')->getDescription('unknown'));
    }

    /** @test **/
    public function can_get_alert_types_of_status()
    {
        $this->assertEquals('error', Status::make('novoVisionMiff')->getAlert('declined'));
        $this->assertNull(Status::make('novoVisionMiff')->getAlert('unknown'));
    }
}
